@@ -106,12 +106,18 @@ typedef struct {
   GtkWidget *spinner;
   gboolean loading;
 
-  /* We have to keep a reference on the actual EmpathyAccountWidget, not just
-   * his GtkWidget. It is the only reliable source we can query to know if
+  /* We have to keep a weak reference on the actual EmpathyAccountWidget, not
+   * just its GtkWidget. It is the only reliable source we can query to know if
    * there are any unsaved changes to the currently selected account. We can't
    * look at the account settings because it does not contain everything that
    * can be changed using the EmpathyAccountWidget. For instance, it does not
-   * contain the state of the "Enabled" checkbox. */
+   * contain the state of the "Enabled" checkbox.
+   *
+   * Even if we create it ourself, we just get a weak ref and not a strong one
+   * as EmpathyAccountWidget unrefs itself when the GtkWidget is destroyed.
+   * That's kinda ugly; cf bgo #640417.
+   *
+   * */
   EmpathyAccountWidget *setting_widget_object;
 
   gboolean  connecting_show;
@@ -485,9 +491,16 @@ account_dialog_create_settings_widget (EmpathyAccountsDialog *dialog,
   const gchar               *icon_name;
   TpAccount                 *account;
 
+  if (priv->setting_widget_object != NULL)
+    g_object_remove_weak_pointer (G_OBJECT (priv->setting_widget_object),
+        (gpointer *) &priv->setting_widget_object);
+
   priv->setting_widget_object =
       empathy_account_widget_new_for_protocol (settings, FALSE);
 
+  g_object_add_weak_pointer (G_OBJECT (priv->setting_widget_object),
+      (gpointer *) &priv->setting_widget_object);
+
   if (accounts_dialog_has_valid_accounts (dialog))
     empathy_account_widget_set_other_accounts_exist (
         priv->setting_widget_object, TRUE);
@@ -1025,8 +1038,22 @@ accounts_dialog_row_changed_foreach (GtkTreeModel *model,
     GtkTreeIter *iter,
     gpointer user_data)
 {
-  gtk_tree_model_row_changed (model, path, iter);
+  TpAccount *account;
 
+  gtk_tree_model_get (model, iter, COL_ACCOUNT, &account, -1);
+
+  if (account == NULL)
+    return FALSE;
+
+  if (tp_account_get_connection_status (account, NULL) ==
+      TP_CONNECTION_STATUS_CONNECTING)
+    {
+      /* Only update the row where we have a connecting account as that's the
+       * ones having a blinking icon. */
+      gtk_tree_model_row_changed (model, path, iter);
+    }
+
+  g_object_unref (account);
   return FALSE;
 }
 
@@ -1872,7 +1899,20 @@ accounts_dialog_set_selected_account (EmpathyAccountsDialog *dialog,
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
   if (accounts_dialog_get_account_iter (dialog, account, &iter))
-    gtk_tree_selection_select_iter (selection, &iter);
+    {
+      GtkTreePath *path;
+      GtkTreeModel *model;
+
+      gtk_tree_selection_select_iter (selection, &iter);
+
+      model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+      path = gtk_tree_model_get_path (model, &iter);
+
+      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (priv->treeview), path, NULL,
+          TRUE, 0, 0.5);
+
+      gtk_tree_path_free (path);
+    }
 }
 
 static void
