@@ -29,15 +29,17 @@
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/simple-approver.h>
 
-#include <libempathy/empathy-dispatcher.h>
+#include <telepathy-yell/telepathy-yell.h>
+
 #include <libempathy/empathy-idle.h>
+
+#include <libempathy/empathy-channel-factory.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-tp-chat.h>
-#include <libempathy/empathy-tp-call.h>
+#include <libempathy/empathy-tp-streamed-media.h>
 #include <libempathy/empathy-tp-file.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-gsettings.h>
 
 #include <extensions/extensions.h>
@@ -72,11 +74,12 @@ typedef struct {
   GtkWidget *dialog;
   /* Channel of the CDO that will be used during the approval */
   TpChannel *main_channel;
+  gboolean auto_approved;
 } EventManagerApproval;
 
 typedef struct {
-  EmpathyDispatcher *dispatcher;
   TpBaseClient *approver;
+  TpBaseClient *auth_approver;
   EmpathyContactManager *contact_manager;
   GSList *events;
   /* Ongoing approvals */
@@ -163,10 +166,8 @@ event_free (EventPriv *event)
   if (event->autoremove_timeout_id != 0)
     g_source_remove (event->autoremove_timeout_id);
 
-  if (event->public.contact)
-    {
-      g_object_unref (event->public.contact);
-    }
+  tp_clear_object (&(event->public.contact));
+  tp_clear_object (&(event->public.account));
 
   g_slice_free (EventPriv, event);
 }
@@ -191,8 +192,25 @@ autoremove_event_timeout_cb (EventPriv *event)
   return FALSE;
 }
 
+static gboolean
+display_notify_area (void)
+
+{
+  GSettings *gsettings;
+  gboolean result;
+
+  gsettings = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
+
+  result = g_settings_get_boolean (gsettings,
+      EMPATHY_PREFS_UI_EVENTS_NOTIFY_AREA);
+  g_object_unref (gsettings);
+
+  return result;
+}
+
 static void
 event_manager_add (EmpathyEventManager *manager,
+    TpAccount *account,
     EmpathyContact *contact,
     EmpathyEventType type,
     const gchar *icon_name,
@@ -206,6 +224,7 @@ event_manager_add (EmpathyEventManager *manager,
   EventPriv               *event;
 
   event = g_slice_new0 (EventPriv);
+  event->public.account = account != NULL ? g_object_ref (account) : NULL;
   event->public.contact = contact ? g_object_ref (contact) : NULL;
   event->public.type = type;
   event->public.icon_name = g_strdup (icon_name);
@@ -220,6 +239,17 @@ event_manager_add (EmpathyEventManager *manager,
 
   DEBUG ("Adding event %p", event);
   priv->events = g_slist_prepend (priv->events, event);
+
+  if (!display_notify_area ())
+    {
+      /* Don't fire the 'event-added' signal as we activate the event now */
+      if (approval != NULL)
+        approval->auto_approved = TRUE;
+
+      empathy_event_activate (&event->public);
+      return;
+    }
+
   g_signal_emit (event->manager, signals[EVENT_ADDED], 0, event);
 
   if (!event->public.must_ack)
@@ -278,10 +308,16 @@ handle_with_time_cb (GObject *source,
 static void
 event_manager_approval_approve (EventManagerApproval *approval)
 {
-  gint64 timestamp = gtk_get_current_event_time ();
+  gint64 timestamp;
 
-  if (timestamp == GDK_CURRENT_TIME)
-    timestamp = EMPATHY_DISPATCHER_CURRENT_TIME;
+  if (approval->auto_approved)
+    {
+      timestamp = TP_USER_ACTION_TIME_NOT_USER_ACTION;
+    }
+  else
+    {
+      timestamp = tp_user_action_time_from_x11 (gtk_get_current_event_time ());
+    }
 
   g_assert (approval->operation != NULL);
 
@@ -299,9 +335,9 @@ static void
 event_text_channel_process_func (EventPriv *event)
 {
   EmpathyTpChat *tp_chat;
-  gint64 timestamp = gtk_get_current_event_time ();
-  if (timestamp == GDK_CURRENT_TIME)
-    timestamp = EMPATHY_DISPATCHER_CURRENT_TIME;
+  gint64 timestamp;
+
+  timestamp = tp_user_action_time_from_x11 (gtk_get_current_event_time ());
 
   if (event->approval->handler != 0)
     {
@@ -367,25 +403,71 @@ reject_channel_claim_cb (GObject *source,
       goto out;
     }
 
-  if (EMPATHY_IS_TP_CALL (user_data))
+  if (EMPATHY_IS_TP_STREAMED_MEDIA (user_data))
     {
-      empathy_tp_call_close (user_data);
+      empathy_tp_streamed_media_close (user_data);
+    }
+  else if (TPY_IS_CALL_CHANNEL (user_data))
+    {
+      tpy_call_channel_hangup_async (user_data,
+          TPY_CALL_STATE_CHANGE_REASON_USER_REQUESTED,
+          "", "", NULL, NULL);
+      tp_channel_close_async (user_data, NULL, NULL);
     }
   else if (EMPATHY_IS_TP_CHAT (user_data))
     {
       empathy_tp_chat_leave (user_data);
     }
+  else if (EMPATHY_IS_TP_FILE (user_data))
+    {
+      empathy_tp_file_close (user_data);
+    }
 
 out:
   g_object_unref (user_data);
 }
 
+static void
+reject_auth_channel_claim_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  TpChannelDispatchOperation *cdo = TP_CHANNEL_DISPATCH_OPERATION (source);
+  GError *error = NULL;
+
+  if (!tp_channel_dispatch_operation_claim_finish (cdo, result, &error))
+    {
+      DEBUG ("Failed to claim channel: %s", error->message);
+
+      g_error_free (error);
+      return;
+    }
+
+  tp_cli_channel_call_close (TP_CHANNEL (user_data), -1,
+      NULL, NULL, NULL, NULL);
+}
+
 static void
 reject_approval (EventManagerApproval *approval)
 {
   /* We have to claim the channel before closing it */
-  tp_channel_dispatch_operation_claim_async (approval->operation,
-      reject_channel_claim_cb, g_object_ref (approval->handler_instance));
+
+  /* Unfortunately, we need to special case the auth channels for the
+   * time being as they don't have a wrapper object handler in
+   * approval->handler_instance as they're not actually handled by
+   * this process, so we can just use a noddy callback to call Close()
+   * directly. */
+  if (approval->handler_instance != NULL)
+    {
+      tp_channel_dispatch_operation_claim_async (approval->operation,
+          reject_channel_claim_cb, g_object_ref (approval->handler_instance));
+    }
+  else if (tp_channel_get_channel_type_id (approval->main_channel)
+      == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
+    {
+      tp_channel_dispatch_operation_claim_async (approval->operation,
+          reject_auth_channel_claim_cb, approval->main_channel);
+    }
 }
 
 static void
@@ -413,8 +495,8 @@ event_channel_process_voip_func (EventPriv *event)
   GtkWidget *dialog;
   GtkWidget *button;
   GtkWidget *image;
-  EmpathyTpCall *call;
   gboolean video;
+  gchar *title;
 
   if (event->approval->dialog != NULL)
     {
@@ -422,9 +504,23 @@ event_channel_process_voip_func (EventPriv *event)
       return;
     }
 
-  call = EMPATHY_TP_CALL (event->approval->handler_instance);
-
-  video = empathy_tp_call_has_initial_video (call);
+  if (event->public.type == EMPATHY_EVENT_TYPE_VOIP)
+    {
+      EmpathyTpStreamedMedia *call;
+      call = EMPATHY_TP_STREAMED_MEDIA (event->approval->handler_instance);
+      video = empathy_tp_streamed_media_has_initial_video (call);
+    }
+  else if (event->public.type == EMPATHY_EVENT_TYPE_CALL)
+    {
+      TpyCallChannel *call;
+      call = TPY_CALL_CHANNEL (event->approval->handler_instance);
+      g_object_get (G_OBJECT (call), "initial-video", &video, NULL);
+    }
+  else
+    {
+      g_warning ("Unknown event type: %d", event->public.type);
+      return;
+    }
 
   dialog = gtk_message_dialog_new (NULL, 0,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
@@ -434,7 +530,13 @@ event_channel_process_voip_func (EventPriv *event)
     GTK_MESSAGE_DIALOG (dialog), video ?
       _("%s is video calling you. Do you want to answer?"):
       _("%s is calling you. Do you want to answer?"),
-      empathy_contact_get_name (event->approval->contact));
+      empathy_contact_get_alias (event->approval->contact));
+
+  title = g_strdup_printf (_("Incoming call from %s"),
+      empathy_contact_get_alias (event->approval->contact));
+
+  gtk_window_set_title (GTK_WINDOW (dialog), title);
+  g_free (title);
 
   /* Set image of the dialog */
   if (video)
@@ -492,7 +594,7 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
   event = event_lookup_by_approval (approval->manager, approval);
 
   sender = empathy_message_get_sender (message);
-  header = empathy_contact_get_name (sender);
+  header = empathy_contact_get_alias (sender);
   msg = empathy_message_get_body (message);
 
   channel = empathy_tp_chat_get_channel (tp_chat);
@@ -501,9 +603,9 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
     event_update (approval->manager, event, EMPATHY_IMAGE_NEW_MESSAGE, header,
         msg);
   else
-    event_manager_add (approval->manager, sender, EMPATHY_EVENT_TYPE_CHAT,
-        EMPATHY_IMAGE_NEW_MESSAGE, header, msg, approval,
-        event_text_channel_process_func, NULL);
+    event_manager_add (approval->manager, NULL, sender,
+        EMPATHY_EVENT_TYPE_CHAT, EMPATHY_IMAGE_NEW_MESSAGE, header, msg,
+        approval, event_text_channel_process_func, NULL);
 
   empathy_sound_play (window, EMPATHY_SOUND_CONVERSATION_NEW);
 
@@ -522,7 +624,8 @@ event_manager_approval_done (EventManagerApproval *approval)
 
       channel_type = tp_channel_get_channel_type_id (approval->main_channel);
 
-      if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
+      if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA ||
+          channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL)
         {
           priv->ringing--;
           if (priv->ringing == 0)
@@ -559,24 +662,39 @@ cdo_invalidated_cb (TpProxy *cdo,
 }
 
 static void
-event_manager_media_channel_got_contact (EventManagerApproval *approval)
+event_manager_call_channel_got_contact_cb (TpConnection *connection,
+                                 EmpathyContact *contact,
+                                 const GError *error,
+                                 gpointer user_data,
+                                 GObject *object)
 {
+  EventManagerApproval *approval = (EventManagerApproval *) user_data;
   EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
-  GtkWidget *window = empathy_main_window_dup ();
+  GtkWidget *window;
+  TpyCallChannel *call;
   gchar *header;
-  EmpathyTpCall *call;
   gboolean video;
 
-  call = EMPATHY_TP_CALL (approval->handler_instance);
+  if (error != NULL)
+    {
+      DEBUG ("Can't get the contact for the call.. Rejecting?");
+      reject_approval (approval);
+      return;
+    }
 
-  video = empathy_tp_call_has_initial_video (call);
+  window = empathy_main_window_dup ();
+  approval->contact = g_object_ref (contact);
+
+  call = TPY_CALL_CHANNEL (approval->handler_instance);
+
+  g_object_get (G_OBJECT (call), "initial-video", &video, NULL);
 
   header = g_strdup_printf (
     video ? _("Incoming video call from %s") :_("Incoming call from %s"),
-    empathy_contact_get_name (approval->contact));
+    empathy_contact_get_alias (approval->contact));
 
-  event_manager_add (approval->manager, approval->contact,
-      EMPATHY_EVENT_TYPE_VOIP,
+  event_manager_add (approval->manager, NULL,
+      approval->contact, EMPATHY_EVENT_TYPE_CALL,
       video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
       header, NULL, approval,
       event_channel_process_voip_func, NULL);
@@ -592,7 +710,40 @@ event_manager_media_channel_got_contact (EventManagerApproval *approval)
 }
 
 static void
-event_manager_media_channel_contact_changed_cb (EmpathyTpCall *call,
+event_manager_media_channel_got_contact (EventManagerApproval *approval)
+{
+  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
+  GtkWidget *window = empathy_main_window_dup ();
+  gchar *header;
+  EmpathyTpStreamedMedia *call;
+  gboolean video;
+
+  call = EMPATHY_TP_STREAMED_MEDIA (approval->handler_instance);
+
+  video = empathy_tp_streamed_media_has_initial_video (call);
+
+  header = g_strdup_printf (
+    video ? _("Incoming video call from %s") :_("Incoming call from %s"),
+    empathy_contact_get_alias (approval->contact));
+
+  event_manager_add (approval->manager, NULL,
+      approval->contact, EMPATHY_EVENT_TYPE_VOIP,
+      video ? EMPATHY_IMAGE_VIDEO_CALL : EMPATHY_IMAGE_VOIP,
+      header, NULL, approval,
+      event_channel_process_voip_func, NULL);
+
+  g_free (header);
+
+  priv->ringing++;
+  if (priv->ringing == 1)
+    empathy_sound_start_playing (window,
+        EMPATHY_SOUND_PHONE_INCOMING, MS_BETWEEN_RING);
+
+  g_object_unref (window);
+}
+
+static void
+event_manager_media_channel_contact_changed_cb (EmpathyTpStreamedMedia *call,
   GParamSpec *param, EventManagerApproval *approval)
 {
   EmpathyContact *contact;
@@ -639,6 +790,7 @@ event_room_channel_process_func (EventPriv *event)
 {
   GtkWidget *dialog, *button, *image;
   TpChannel *channel = event->approval->main_channel;
+  gchar *title;
 
   if (event->approval->dialog != NULL)
     {
@@ -650,9 +802,15 @@ event_room_channel_process_func (EventPriv *event)
   dialog = gtk_message_dialog_new (NULL, 0,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _("Room invitation"));
 
+  title = g_strdup_printf (_("Invitation to join %s"),
+      tp_channel_get_identifier (channel));
+
+  gtk_window_set_title (GTK_WINDOW (dialog), title);
+  g_free (title);
+
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
       _("%s is inviting you to join %s"),
-      empathy_contact_get_name (event->approval->contact),
+      empathy_contact_get_alias (event->approval->contact),
       tp_channel_get_identifier (channel));
 
   gtk_dialog_set_default_response (GTK_DIALOG (dialog),
@@ -676,6 +834,41 @@ event_room_channel_process_func (EventPriv *event)
   event->approval->dialog = dialog;
 }
 
+static void
+display_invite_room_dialog (EventManagerApproval *approval)
+{
+  GtkWidget *window = empathy_main_window_dup ();
+  const gchar *invite_msg;
+  gchar *msg;
+  TpHandle self_handle;
+
+  self_handle = tp_channel_group_get_self_handle (approval->main_channel);
+  tp_channel_group_get_local_pending_info (approval->main_channel, self_handle,
+      NULL, NULL, &invite_msg);
+
+  if (approval->contact != NULL)
+    {
+      msg = g_strdup_printf (_("%s invited you to join %s"),
+          empathy_contact_get_alias (approval->contact),
+          tp_channel_get_identifier (approval->main_channel));
+    }
+  else
+    {
+      msg = g_strdup_printf (_("You have been invited to join %s"),
+          tp_channel_get_identifier (approval->main_channel));
+    }
+
+  event_manager_add (approval->manager, NULL,
+      approval->contact, EMPATHY_EVENT_TYPE_INVITATION,
+      EMPATHY_IMAGE_GROUP_MESSAGE, msg, invite_msg, approval,
+      event_room_channel_process_func, NULL);
+
+  empathy_sound_play (window, EMPATHY_SOUND_CONVERSATION_NEW);
+
+  g_free (msg);
+  g_object_unref (window);
+}
+
 static void
 event_manager_muc_invite_got_contact_cb (TpConnection *connection,
                                          EmpathyContact *contact,
@@ -684,36 +877,17 @@ event_manager_muc_invite_got_contact_cb (TpConnection *connection,
                                          GObject *object)
 {
   EventManagerApproval *approval = (EventManagerApproval *) user_data;
-  GtkWidget *window = empathy_main_window_dup ();
-  const gchar *invite_msg;
-  gchar *msg;
-  TpHandle self_handle;
 
   if (error != NULL)
     {
-      /* FIXME: We should probably still display the event */
       DEBUG ("Error: %s", error->message);
-      return;
+    }
+  else
+    {
+      approval->contact = g_object_ref (contact);
     }
 
-  approval->contact = g_object_ref (contact);
-
-  self_handle = tp_channel_group_get_self_handle (approval->main_channel);
-  tp_channel_group_get_local_pending_info (approval->main_channel, self_handle,
-      NULL, NULL, &invite_msg);
-
-  msg = g_strdup_printf (_("%s invited you to join %s"),
-      empathy_contact_get_name (approval->contact),
-      tp_channel_get_identifier (approval->main_channel));
-
-  event_manager_add (approval->manager, approval->contact,
-      EMPATHY_EVENT_TYPE_CHAT, EMPATHY_IMAGE_GROUP_MESSAGE, msg, invite_msg,
-      approval, event_room_channel_process_func, NULL);
-
-  empathy_sound_play (window, EMPATHY_SOUND_CONVERSATION_NEW);
-
-  g_free (msg);
-  g_object_unref (window);
+  display_invite_room_dialog (approval);
 }
 
 static void
@@ -730,10 +904,11 @@ event_manager_ft_got_contact_cb (TpConnection *connection,
   approval->contact = g_object_ref (contact);
 
   header = g_strdup_printf (_("Incoming file transfer from %s"),
-                            empathy_contact_get_name (approval->contact));
+                            empathy_contact_get_alias (approval->contact));
 
-  event_manager_add (approval->manager, approval->contact,
-      EMPATHY_EVENT_TYPE_TRANSFER, EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL,
+  event_manager_add (approval->manager, NULL,
+      approval->contact, EMPATHY_EVENT_TYPE_TRANSFER,
+      EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL,
       approval, event_channel_process_func, NULL);
 
   /* FIXME better sound for incoming file transfers ?*/
@@ -743,8 +918,14 @@ event_manager_ft_got_contact_cb (TpConnection *connection,
   g_object_unref (window);
 }
 
-/* If there is a file-transfer or media channel consider it as the
- * main one. */
+static void
+event_manager_auth_process_func (EventPriv *event)
+{
+  empathy_event_approve ((EmpathyEvent *) event);
+}
+
+/* If there is a file-transfer, media, or auth channel consider it as
+ * the main one. */
 static TpChannel *
 find_main_channel (GList *channels)
 {
@@ -762,7 +943,9 @@ find_main_channel (GList *channels)
       channel_type = tp_channel_get_channel_type_id (channel);
 
       if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA ||
-          channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
+          channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL ||
+          channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER ||
+          channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
         return channel;
 
       else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_TEXT)
@@ -811,7 +994,7 @@ approve_channels (TpSimpleApprover *approver,
     {
       EmpathyTpChat *tp_chat;
 
-      tp_chat = empathy_tp_chat_new (channel);
+      tp_chat = empathy_tp_chat_new (account, channel);
       approval->handler_instance = G_OBJECT (tp_chat);
 
       if (tp_proxy_has_interface (channel, TP_IFACE_CHANNEL_INTERFACE_GROUP))
@@ -825,9 +1008,16 @@ approve_channels (TpSimpleApprover *approver,
               DEBUG ("Have been invited to %s. Ask user if he wants to accept",
                   tp_channel_get_identifier (channel));
 
-              empathy_tp_contact_factory_get_from_handle (connection,
-                  inviter, event_manager_muc_invite_got_contact_cb,
-                  approval, NULL, G_OBJECT (self));
+              if (inviter != 0)
+                {
+                  empathy_tp_contact_factory_get_from_handle (connection,
+                      inviter, event_manager_muc_invite_got_contact_cb,
+                      approval, NULL, G_OBJECT (self));
+                }
+              else
+                {
+                  display_invite_room_dialog (approval);
+                }
 
               goto out;
             }
@@ -835,6 +1025,7 @@ approve_channels (TpSimpleApprover *approver,
           /* We are not invited, approve the channel right now */
           tp_add_dispatch_operation_context_accept (context);
 
+          approval->auto_approved = TRUE;
           event_manager_approval_approve (approval);
           return;
         }
@@ -846,7 +1037,8 @@ approve_channels (TpSimpleApprover *approver,
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
     {
       EmpathyContact *contact;
-      EmpathyTpCall *call = empathy_tp_call_new (channel);
+      EmpathyTpStreamedMedia *call = empathy_tp_streamed_media_new (account,
+        channel);
 
       approval->handler_instance = G_OBJECT (call);
 
@@ -865,16 +1057,38 @@ approve_channels (TpSimpleApprover *approver,
         }
 
     }
+  else if (channel_type == TPY_IFACE_QUARK_CHANNEL_TYPE_CALL)
+    {
+      TpyCallChannel *call = TPY_CALL_CHANNEL (channel);
+      const gchar *id;
+
+      approval->handler_instance = g_object_ref (call);
+
+      id = tp_channel_get_identifier (channel);
+
+      empathy_tp_contact_factory_get_from_id (connection, id,
+        event_manager_call_channel_got_contact_cb,
+        approval, NULL, G_OBJECT (self));
+    }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
     {
       TpHandle handle;
+      EmpathyTpFile *tp_file = empathy_tp_file_new (channel);
+
+      approval->handler_instance = G_OBJECT (tp_file);
 
       handle = tp_channel_get_handle (channel, NULL);
 
-      connection = tp_channel_borrow_connection (channel);
       empathy_tp_contact_factory_get_from_handle (connection, handle,
         event_manager_ft_got_contact_cb, approval, NULL, G_OBJECT (self));
     }
+  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_SERVER_AUTHENTICATION)
+    {
+      event_manager_add (approval->manager, account, NULL, EMPATHY_EVENT_TYPE_AUTH,
+          GTK_STOCK_DIALOG_AUTHENTICATION, tp_account_get_display_name (account),
+          _("Password required"), approval,
+          event_manager_auth_process_func, NULL);
+    }
   else
     {
       GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
@@ -926,15 +1140,16 @@ event_manager_pendings_changed_cb (EmpathyContactList  *list,
       return;
     }
 
-  header = g_strdup_printf (_("Subscription requested by %s"),
-    empathy_contact_get_name (contact));
+  header = g_strdup_printf (
+      _("%s would like permission to see when you are online"),
+      empathy_contact_get_alias (contact));
 
   if (!EMP_STR_EMPTY (message))
     event_msg = g_strdup_printf (_("\nMessage: %s"), message);
   else
     event_msg = NULL;
 
-  event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_SUBSCRIPTION,
+  event_manager_add (manager, NULL, contact, EMPATHY_EVENT_TYPE_SUBSCRIPTION,
       GTK_STOCK_DIALOG_QUESTION, header, event_msg, NULL,
       event_pending_subscribe_func, NULL);
 
@@ -973,11 +1188,12 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
           if (g_settings_get_boolean (gsettings,
                 EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNOUT))
             {
-              header = g_strdup_printf (_("%s is now offline."),
-                  empathy_contact_get_name (contact));
+              header = g_strdup_printf ("<b>%s</b>",
+                  empathy_contact_get_alias (contact));
 
-              event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
-                  EMPATHY_IMAGE_AVATAR_DEFAULT, header, NULL, NULL, NULL, NULL);
+              event_manager_add (manager, NULL, contact,
+                  EMPATHY_EVENT_TYPE_PRESENCE, EMPATHY_IMAGE_AVATAR_DEFAULT,
+                  header, _("Disconnected"), NULL, NULL, NULL);
             }
         }
     }
@@ -993,11 +1209,12 @@ event_manager_presence_changed_cb (EmpathyContact *contact,
           if (g_settings_get_boolean (gsettings,
                 EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNIN))
             {
-              header = g_strdup_printf (_("%s is now online."),
-                  empathy_contact_get_name (contact));
+              header = g_strdup_printf ("<b>%s</b>",
+                  empathy_contact_get_alias (contact));
 
-              event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
-                  EMPATHY_IMAGE_AVATAR_DEFAULT, header, NULL, NULL, NULL, NULL);
+              event_manager_add (manager, NULL, contact,
+                  EMPATHY_EVENT_TYPE_PRESENCE, EMPATHY_IMAGE_AVATAR_DEFAULT,
+                  header, _("Connected"), NULL, NULL, NULL);
             }
         }
     }
@@ -1059,8 +1276,8 @@ event_manager_finalize (GObject *object)
   g_slist_foreach (priv->approvals, (GFunc) event_manager_approval_free, NULL);
   g_slist_free (priv->approvals);
   g_object_unref (priv->contact_manager);
-  g_object_unref (priv->dispatcher);
   g_object_unref (priv->approver);
+  g_object_unref (priv->auth_approver);
 }
 
 static void
@@ -1099,7 +1316,6 @@ empathy_event_manager_class_init (EmpathyEventManagerClass *klass)
       g_cclosure_marshal_VOID__POINTER,
       G_TYPE_NONE, 1, G_TYPE_POINTER);
 
-
   g_type_class_add_private (object_class, sizeof (EmpathyEventManagerPriv));
 }
 
@@ -1109,11 +1325,11 @@ empathy_event_manager_init (EmpathyEventManager *manager)
   EmpathyEventManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
     EMPATHY_TYPE_EVENT_MANAGER, EmpathyEventManagerPriv);
   TpDBusDaemon *dbus;
+  EmpathyChannelFactory *factory;
   GError *error = NULL;
 
   manager->priv = priv;
 
-  priv->dispatcher = empathy_dispatcher_dup_singleton ();
   priv->contact_manager = empathy_contact_manager_dup_singleton ();
   g_signal_connect (priv->contact_manager, "pendings-changed",
     G_CALLBACK (event_manager_pendings_changed_cb), manager);
@@ -1132,6 +1348,11 @@ empathy_event_manager_init (EmpathyEventManager *manager)
   priv->approver = tp_simple_approver_new (dbus, "Empathy.EventManager", FALSE,
       approve_channels, manager, NULL);
 
+  factory = empathy_channel_factory_new ();
+  tp_base_client_set_channel_factory (priv->approver,
+      TP_CLIENT_CHANNEL_FACTORY (factory));
+  g_object_unref (factory);
+
   /* Private text channels */
   tp_base_client_take_approver_filter (priv->approver,
       tp_asv_new (
@@ -1161,6 +1382,35 @@ empathy_event_manager_init (EmpathyEventManager *manager)
           TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
         TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
         NULL));
+  tp_base_client_take_approver_filter (priv->approver,
+      tp_asv_new (
+        TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
+          TPY_IFACE_CHANNEL_TYPE_CALL,
+        TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_CONTACT,
+        NULL));
+
+  /* I don't feel good about doing this, and I'm sorry, but the
+   * capabilities connection feature is added earlier because it's
+   * needed for EmpathyTpChat. If the capabilities feature is required
+   * then preparing an auth channel (which of course appears in the
+   * CONNECTING state) will never be prepared. So the options are
+   * either to create another approver like I've done, or to port
+   * EmpathyTpChat and its users to not depend on the connection being
+   * prepared with capabilities. I chose the former, obviously. :-) */
+
+  priv->auth_approver = tp_simple_approver_new (dbus,
+      "Empathy.AuthEventManager", FALSE, approve_channels, manager,
+      NULL);
+
+  /* SASL auth channels */
+  tp_base_client_take_approver_filter (priv->auth_approver,
+      tp_asv_new (
+        TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING,
+          TP_IFACE_CHANNEL_TYPE_SERVER_AUTHENTICATION,
+        TP_PROP_CHANNEL_TYPE_SERVER_AUTHENTICATION_AUTHENTICATION_METHOD,
+          G_TYPE_STRING,
+          TP_IFACE_CHANNEL_INTERFACE_SASL_AUTHENTICATION,
+        NULL));
 
   if (!tp_base_client_register (priv->approver, &error))
     {
@@ -1168,6 +1418,12 @@ empathy_event_manager_init (EmpathyEventManager *manager)
       g_error_free (error);
     }
 
+  if (!tp_base_client_register (priv->auth_approver, &error))
+    {
+      DEBUG ("Failed to register auth Approver: %s", error->message);
+      g_error_free (error);
+    }
+
   g_object_unref (dbus);
 }
 
@@ -1219,3 +1475,23 @@ empathy_event_inhibit_updates (EmpathyEvent *event_public)
 
   event->inhibit = TRUE;
 }
+
+void
+empathy_event_approve (EmpathyEvent *event_public)
+{
+  EventPriv *event = (EventPriv *) event_public;
+
+  g_return_if_fail (event_public != NULL);
+
+  event_manager_approval_approve (event->approval);
+}
+
+void
+empathy_event_decline (EmpathyEvent *event_public)
+{
+  EventPriv *event = (EventPriv *) event_public;
+
+  g_return_if_fail (event_public != NULL);
+
+  reject_approval (event->approval);
+}
