@@ -40,7 +40,6 @@
 #include <libempathy/empathy-tp-file.h>
 #include <libempathy/empathy-utils.h>
 
-#include <libempathy-gtk/empathy-conf.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-geometry.h>
 #include <libempathy-gtk/empathy-images.h>
@@ -66,8 +65,6 @@ typedef struct {
   GtkWidget *open_button;
   GtkWidget *abort_button;
   GtkWidget *clear_button;
-
-  guint save_geometry_id;
 } EmpathyFTManagerPriv;
 
 enum
@@ -208,7 +205,7 @@ ft_manager_remove_file_from_model (EmpathyFTManager *manager,
   g_return_if_fail (row_ref);
 
   DEBUG ("Removing file transfer from window: contact=%s, filename=%s",
-      empathy_contact_get_name (empathy_ft_handler_get_contact (handler)),
+      empathy_contact_get_alias (empathy_ft_handler_get_contact (handler)),
       empathy_ft_handler_get_filename (handler));
 
   /* Get the iter from the row_ref */
@@ -299,7 +296,7 @@ ft_manager_format_contact_info (EmpathyFTHandler *handler)
   char *retval;
 
   incoming = empathy_ft_handler_is_incoming (handler);
-  contact_name = empathy_contact_get_name
+  contact_name = empathy_contact_get_alias
     (empathy_ft_handler_get_contact (handler));
   filename = empathy_ft_handler_get_filename (handler);
 
@@ -329,7 +326,7 @@ ft_manager_format_error_message (EmpathyFTHandler *handler,
 
   contact = empathy_ft_handler_get_contact (handler);
   if (contact)
-    contact_name = empathy_contact_get_name (contact);
+    contact_name = empathy_contact_get_alias (contact);
 
   filename = empathy_ft_handler_get_filename (handler);
 
@@ -481,7 +478,7 @@ do_real_transfer_done (EmpathyFTManager *manager,
   g_return_if_fail (row_ref != NULL);
 
   incoming = empathy_ft_handler_is_incoming (handler);
-  contact_name = empathy_contact_get_name
+  contact_name = empathy_contact_get_alias
     (empathy_ft_handler_get_contact (handler));
   filename = empathy_ft_handler_get_filename (handler);
 
@@ -860,7 +857,7 @@ ft_manager_stop (EmpathyFTManager *manager)
   g_return_if_fail (handler != NULL);
 
   DEBUG ("Stopping file transfer: contact=%s, filename=%s",
-      empathy_contact_get_name (empathy_ft_handler_get_contact (handler)),
+      empathy_contact_get_alias (empathy_ft_handler_get_contact (handler)),
       empathy_ft_handler_get_filename (handler));
 
   empathy_ft_handler_cancel_transfer (handler);
@@ -868,38 +865,6 @@ ft_manager_stop (EmpathyFTManager *manager)
   g_object_unref (handler);
 }
 
-static gboolean
-ft_manager_save_geometry_timeout_cb (EmpathyFTManager *manager)
-{
-  EmpathyFTManagerPriv *priv = GET_PRIV (manager);
-  gint x, y, w, h;
-
-  gtk_window_get_size (GTK_WINDOW (priv->window), &w, &h);
-  gtk_window_get_position (GTK_WINDOW (priv->window), &x, &y);
-
-  empathy_geometry_save ("ft-manager", x, y, w, h);
-
-  priv->save_geometry_id = 0;
-
-  return FALSE;
-}
-
-static gboolean
-ft_manager_configure_event_cb (GtkWidget *widget,
-                               GdkEventConfigure *event,
-                               EmpathyFTManager *manager)
-{
-  EmpathyFTManagerPriv *priv = GET_PRIV (manager);
-
-  if (priv->save_geometry_id != 0)
-    g_source_remove (priv->save_geometry_id);
-
-  priv->save_geometry_id = g_timeout_add (500,
-      (GSourceFunc) ft_manager_save_geometry_timeout_cb, manager);
-
-  return FALSE;
-}
-
 static void
 ft_manager_response_cb (GtkWidget *widget,
                         gint response,
@@ -916,6 +881,12 @@ ft_manager_response_cb (GtkWidget *widget,
       case RESPONSE_STOP:
         ft_manager_stop (manager);
         break;
+      case GTK_RESPONSE_NONE:
+      case GTK_RESPONSE_DELETE_EVENT:
+        /* Do nothing */
+        break;
+      default:
+        g_assert_not_reached ();
     }
 }
 
@@ -970,7 +941,6 @@ static void
 ft_manager_build_ui (EmpathyFTManager *manager)
 {
   GtkBuilder *gui;
-  gint x, y, w, h;
   GtkTreeView *view;
   GtkListStore *liststore;
   GtkTreeViewColumn *column;
@@ -993,28 +963,13 @@ ft_manager_build_ui (EmpathyFTManager *manager)
       "ft_manager_dialog", "destroy", ft_manager_destroy_cb,
       "ft_manager_dialog", "response", ft_manager_response_cb,
       "ft_manager_dialog", "delete-event", ft_manager_delete_event_cb,
-      "ft_manager_dialog", "configure-event", ft_manager_configure_event_cb,
       "ft_manager_dialog", "key-press-event", ft_manager_key_press_event_cb,
       NULL);
 
   empathy_builder_unref_and_keep_widget (gui, priv->window);
 
   /* Window geometry. */
-  empathy_geometry_load ("ft-manager", &x, &y, &w, &h);
-
-  if (x >= 0 && y >= 0)
-    {
-      /* Let the window manager position it if we don't have
-       * good x, y coordinates. */
-      gtk_window_move (GTK_WINDOW (priv->window), x, y);
-    }
-
-  if (w > 0 && h > 0)
-    {
-      /* Use the defaults from the ui file if we don't have
-       * good w, h geometry. */
-      gtk_window_resize (GTK_WINDOW (priv->window), w, h);
-    }
+  empathy_geometry_bind (GTK_WINDOW (priv->window), "ft-manager");
 
   /* Setup the tree view */
   view = GTK_TREE_VIEW (priv->treeview);
@@ -1099,9 +1054,6 @@ empathy_ft_manager_finalize (GObject *object)
 
   g_hash_table_destroy (priv->ft_handler_to_row_ref);
 
-  if (priv->save_geometry_id != 0)
-    g_source_remove (priv->save_geometry_id);
-
   G_OBJECT_CLASS (empathy_ft_manager_parent_class)->finalize (object);
 }
 
